from pyhausbus.de.hausbus.homeassistant.proxy.pT1000.params.ELastEvent import ELastEvent
import pyhausbus.HausBusUtils as HausBusUtils

class EvStatus:
  CLASS_ID = 49
  FUNCTION_ID = 203

  def __init__(self,celsius:int, lastEvent:ELastEvent):
    self.celsius=celsius
    self.lastEvent=lastEvent


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvStatus(HausBusUtils.bytesToWord(dataIn, offset), ELastEvent._fromBytes(dataIn, offset))

  def __str__(self):
    return f"EvStatus(celsius={self.celsius}, lastEvent={self.lastEvent})"

  '''
  @param celsius Grad Celsius.
  '''
  def getCelsius(self):
    return self.celsius

  '''
  @param lastEvent .
  '''
  def getLastEvent(self):
    return self.lastEvent






import pyhausbus.HausBusUtils as HausBusUtils

class GetUnitGroupStatus:
  CLASS_ID = 0
  FUNCTION_ID = 18

  def __init__(self,index:int):
    self.index=index


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return GetUnitGroupStatus(HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"GetUnitGroupStatus(index={self.index})"

  '''
  @param index Index der logischen Gruppe in diesem Controller.
  '''
  def getIndex(self):
    return self.index






from pyhausbus.HomeServer import HomeServer
from pyhausbus.ResultWorker import ResultWorker
from pyhausbus.IBusDataListener import IBusDataListener
from pyhausbus.IBusDeviceListener import IBusDeviceListener
from pyhausbus.de.hausbus.homeassistant.proxy.Controller import Controller
from pyhausbus.de.hausbus.homeassistant.proxy.Taster import Taster
from pyhausbus.de.hausbus.homeassistant.proxy.Led import Led
from pyhausbus.de.hausbus.homeassistant.proxy.Dimmer import Dimmer
from pyhausbus.de.hausbus.homeassistant.proxy.Schalter import Schalter
from pyhausbus.de.hausbus.homeassistant.proxy.controller.data.ModuleId import ModuleId
from pyhausbus.de.hausbus.homeassistant.proxy.controller.data.RemoteObjects import RemoteObjects
from pyhausbus.de.hausbus.homeassistant.proxy.LogicalButton import LogicalButton
from pyhausbus.de.hausbus.homeassistant.proxy.controller.params import EIndex
from pyhausbus.de.hausbus.homeassistant.proxy.taster.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.dimmer.params.EDirection import EDirection
from pyhausbus.de.hausbus.homeassistant.proxy.dimmer.data.EvOn import EvOn
from pyhausbus.de.hausbus.homeassistant.proxy.dimmer.data.EvOff import EvOff
from pyhausbus.de.hausbus.homeassistant.proxy.Feuchtesensor import Feuchtesensor
from pyhausbus.ABusFeature import ABusFeature
import pyhausbus.HausBusUtils
import logging
from pyhausbus.ObjectId import ObjectId
import time
LOGGER = logging.getLogger("pyhausbus")

class Main(IBusDataListener, IBusDeviceListener):

  def __init__(self):
    
    logging.basicConfig(
      level=logging.DEBUG,  # alle Meldungen ab DEBUG
      format="%(asctime)s [%(levelname)s] %(name)s: %(message)s",
      datefmt="%Y-%m-%d %H:%M:%S:%S",  # Format fuer den Timestamp
      filename="mein_log.log",                    # Name der Log-Datei
      filemode="w"                                # 'a' = anhaengen, 'w' = ueberschreiben
    )
    
    '''
    Instantiate Homeserver, add as Lister and search Devices
    Afterwards all devices respond with their moduleId. See method busDataReceived
    '''
    self.server = HomeServer()
    self.server.addBusDeviceListener(self)
    #self.server.addBusEventListener(self)
    self.server.searchDevices()

    ''' Example how to directly create a feature with given class and instance id'''
    #Dimmer.create(22784, 5).setBrightness(100, 0)

    ''' Example how to directly create a feature with given ObjectId and wait for the result '''
    #taster = Taster(1313542180)
    ''' Then we call the method'''
    #taster.getConfiguration()
    ''' And then wait for the Result with a timeout of 2 seconds'''
    #configuration = ResultWorker().waitForResult(2)
    #print("configuration = "+str(configuration))

    #self.doTests()


  def busDataReceived(self, busDataMessage):
      LOGGER.debug("got: " + str(busDataMessage.getData()) + " from " + str(ObjectId(busDataMessage.getSenderObjectId())) + " to " + str(ObjectId(busDataMessage.getReceiverObjectId())))

  def newDeviceDetected(self,device_id:int, model_type: str, module_id: ModuleId, configuration: Configuration, channels: list[ABusFeature]):
      LOGGER.info(f"newDeviceDetected {device_id} model_type {model_type} module_id {module_id} configuration {configuration}")
      for actFeature in channels:
        LOGGER.info(f"channel {actFeature}")

  def doTests(self):

    controller = Controller.create(3359, 1)
    controller.getConfiguration()
    print("Controller.configuration = "+str(ResultWorker().waitForResult(2)))
    controller.getModuleId(EIndex.EIndex.RUNNING)
    print("Controller.moduleId = "+str(ResultWorker().waitForResult(2)))
    controller.ping()
    print("Controller.pong = "+str(ResultWorker().waitForResult(2)))

    dimmer = Dimmer.create(22784, 5)
    dimmer.getConfiguration()
    print("Dimmer.configuration = "+str(ResultWorker().waitForResult(2)))
    dimmer.getStatus()
    print("Dimmer.status = "+str(ResultWorker().waitForResult(2)))
    dimmer.start(EDirection.TO_LIGHT)
    print("Dimmer.evOn = "+str(ResultWorker().waitForEvent(EvOn, dimmer.getObjectId(), 5)))
    dimmer.start(EDirection.TO_DARK)
    print("Dimmer.evOff = "+str(ResultWorker().waitForEvent(EvOff, dimmer.getObjectId(), 5)))

    feuchtesensor = Feuchtesensor.create(25661 , 88)
    feuchtesensor.getConfiguration()
    print("Feuchtesensor.configuration = "+str(ResultWorker().waitForResult(2)))
    feuchtesensor.getStatus()
    print("Feuchtesensor.status = "+str(ResultWorker().waitForResult(2)))

    led = Led.create(20043,54)
    led.getConfiguration()
    print("Led.configuration = "+str(ResultWorker().waitForResult(2)))
    led.getStatus()
    print("Led.status = "+str(ResultWorker().waitForResult(2)))
    led.on(50, 5, 0)
    led.getStatus()
    print("Led.status = "+str(ResultWorker().waitForResult(2)))
    time.sleep(2)
    led.getStatus()
    print("Led.status = "+str(ResultWorker().waitForResult(2)))
    time.sleep(3)
    led.getStatus()
    print("Led.status = "+str(ResultWorker().waitForResult(2)))

Main()



import pyhausbus.HausBusUtils as HausBusUtils

class GetConfiguration:
  CLASS_ID = 36
  FUNCTION_ID = 0

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return GetConfiguration()

  def __str__(self):
    return f"GetConfiguration()"






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EMonth(Enum):
  January=1
  February=2
  March=3
  April=4
  May=5
  June=6
  July=7
  August=8
  September=9
  October=10
  November=11
  December=12
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EMonth.__members__.values():
      if (act.value == checkValue):
        return act

    return EMonth.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EMonth':
    try:
      return EMonth[name]
    except KeyError:
      return EMonth.SER_UNKNOWN 







import pyhausbus.HausBusUtils as HausBusUtils

class GetCurrentIp:
  CLASS_ID = 91
  FUNCTION_ID = 2

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return GetCurrentIp()

  def __str__(self):
    return f"GetCurrentIp()"






from pyhausbus.de.hausbus.homeassistant.proxy.helligkeitssensor.params.ELastEvent import ELastEvent
import pyhausbus.HausBusUtils as HausBusUtils

class Status:
  CLASS_ID = 39
  FUNCTION_ID = 129

  def __init__(self,brightness:int, lastEvent:ELastEvent):
    self.brightness=brightness
    self.lastEvent=lastEvent


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Status(HausBusUtils.bytesToWord(dataIn, offset), ELastEvent._fromBytes(dataIn, offset))

  def __str__(self):
    return f"Status(brightness={self.brightness}, lastEvent={self.lastEvent})"

  '''
  @param brightness Helligkeitswert.
  '''
  def getBrightness(self):
    return self.brightness

  '''
  @param lastEvent .
  '''
  def getLastEvent(self):
    return self.lastEvent






import pyhausbus.HausBusUtils as HausBusUtils


class BusDataMessage:

  def __init__(self, senderObjectId, receiverObjectId, data):
    self.senderObjectId = senderObjectId
    self.receiverObjectId = receiverObjectId
    self.data = data
    self.time = HausBusUtils.getClockIndependMillis()

  def getSenderObjectId(self):
    return self.senderObjectId

  def getReceiverObjectId(self):
    return self.receiverObjectId

  def getData(self):
    return self.data

  def __str__(self):
    return f"BusMessage(senderObjectId={self.senderObjectId}, receiverObjectId={self.receiverObjectId}, data={self.data})"




import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EStatus(Enum):
  OFF=0
  ON=1
  BLINK=2
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EStatus.__members__.values():
      if (act.value == checkValue):
        return act

    return EStatus.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EStatus':
    try:
      return EStatus[name]
    except KeyError:
      return EStatus.SER_UNKNOWN 







from pyhausbus.de.hausbus.homeassistant.proxy.gateway.params.EValue import EValue
import pyhausbus.HausBusUtils as HausBusUtils

class SetPreferLoxone:
  CLASS_ID = 176
  FUNCTION_ID = 2

  def __init__(self,value:EValue):
    self.value=value


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return SetPreferLoxone(EValue._fromBytes(dataIn, offset))

  def __str__(self):
    return f"SetPreferLoxone(value={self.value})"

  '''
  @param value Diese Funktion setzt das Flag \"preferLoxone\" in der Konfiguration entsprechend Persistent..
  '''
  def getValue(self):
    return self.value






import pyhausbus.HausBusUtils as HausBusUtils

class EvNewDeviceId:
  CLASS_ID = 0
  FUNCTION_ID = 201

  def __init__(self,deviceId:int):
    self.deviceId=deviceId


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvNewDeviceId(HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"EvNewDeviceId(deviceId={self.deviceId})"

  '''
  @param deviceId neue ID.
  '''
  def getDeviceId(self):
    return self.deviceId






from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.params.EType import EType
import pyhausbus.HausBusUtils as HausBusUtils

class RegisterConfiguration:
  CLASS_ID = 45
  FUNCTION_ID = 129

  def __init__(self,idx:int, node:int, type:EType, address:int):
    self.idx=idx
    self.node=node
    self.type=type
    self.address=address


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return RegisterConfiguration(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), EType._fromBytes(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"RegisterConfiguration(idx={self.idx}, node={self.node}, type={self.type}, address={self.address})"

  '''
  @param idx index of the configuration slot.
  '''
  def getIdx(self):
    return self.idx

  '''
  @param node device node on ModBus.
  '''
  def getNode(self):
    return self.node

  '''
  @param type Unterstuetzte Register Typen.
  '''
  def getType(self):
    return self.type

  '''
  @param address Register Adresse.
  '''
  def getAddress(self):
    return self.address






from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.daliLine.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.daliLine.params.EErrorCode import EErrorCode

class DaliLine(ABusFeature):
  CLASS_ID:int = 160

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return DaliLine(HausBusUtils.getObjectId(deviceId, 160, instanceId))

  """
  @param address0 .
  @param address1 .
  @param address2 .
  @param address3 .
  """
  def getConfiguration(self, address0:int, address1:int, address2:int, address3:int):
    LOGGER.debug("getConfiguration"+" address0 = "+str(address0)+" address1 = "+str(address1)+" address2 = "+str(address2)+" address3 = "+str(address3))
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    hbCommand.addByte(address0)
    hbCommand.addByte(address1)
    hbCommand.addByte(address2)
    hbCommand.addByte(address3)
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param address0 .
  @param address1 .
  @param address2 .
  @param address3 .
  """
  def setConfiguration(self, address0:int, address1:int, address2:int, address3:int):
    LOGGER.debug("setConfiguration"+" address0 = "+str(address0)+" address1 = "+str(address1)+" address2 = "+str(address2)+" address3 = "+str(address3))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addByte(address0)
    hbCommand.addByte(address1)
    hbCommand.addByte(address2)
    hbCommand.addByte(address3)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def allOff(self):
    LOGGER.debug("allOff")
    hbCommand = HausBusCommand(self.objectId, 2, "allOff")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def allOn(self):
    LOGGER.debug("allOn")
    hbCommand = HausBusCommand(self.objectId, 3, "allOn")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param command Dali Kommando s.Spezifikation.
  @param address Kurz- oder Gruppenadresse YAAA AAAS\r\n64 Kurzadressen           0AAA AAAS\r\n16 Gruppenadressen        100A AAAS\r\nSammelaufruf              1111 111S\r\n\r\nY: Adressenart: Y=?  ? ? ??  ?? ??? ??  ?0?  ? ? ??  ?? ??? ??  ? ? Kurzadresse.
  """
  def sendCommand(self, command:int, address:int):
    LOGGER.debug("sendCommand"+" command = "+str(command)+" address = "+str(address))
    hbCommand = HausBusCommand(self.objectId, 4, "sendCommand")
    hbCommand.addByte(command)
    hbCommand.addByte(address)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param address0 .
  @param address1 .
  @param address2 .
  @param address3 .
  """
  def Configuration(self, address0:int, address1:int, address2:int, address3:int):
    LOGGER.debug("Configuration"+" address0 = "+str(address0)+" address1 = "+str(address1)+" address2 = "+str(address2)+" address3 = "+str(address3))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addByte(address0)
    hbCommand.addByte(address1)
    hbCommand.addByte(address2)
    hbCommand.addByte(address3)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param status .
  """
  def Status(self, status:int):
    LOGGER.debug("Status"+" status = "+str(status))
    hbCommand = HausBusCommand(self.objectId, 129, "Status")
    hbCommand.addByte(status)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





import pyhausbus.HausBusUtils as HausBusUtils

class EvHoldEnd:
  CLASS_ID = 33
  FUNCTION_ID = 204

  def __init__(self,address:int, command:int):
    self.address=address
    self.command=command


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvHoldEnd(HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"EvHoldEnd(address={self.address}, command={self.command})"

  '''
  @param address IR Adresse.
  '''
  def getAddress(self):
    return self.address

  '''
  @param command IR Kommando.
  '''
  def getCommand(self):
    return self.command






from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.params.EErrorCode import EErrorCode
import pyhausbus.HausBusUtils as HausBusUtils

class EvError:
  CLASS_ID = 45
  FUNCTION_ID = 255

  def __init__(self,errorCode:EErrorCode, data:int):
    self.errorCode=errorCode
    self.data=data


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvError(EErrorCode._fromBytes(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"EvError(errorCode={self.errorCode}, data={self.data})"

  '''
  @param errorCode .
  '''
  def getErrorCode(self):
    return self.errorCode

  '''
  @param data .
  '''
  def getData(self):
    return self.data






import pyhausbus.HausBusUtils as HausBusUtils

class EvCmdDelay:
  CLASS_ID = 19
  FUNCTION_ID = 203

  def __init__(self,cmdDelay:int):
    self.cmdDelay=cmdDelay


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvCmdDelay(HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"EvCmdDelay(cmdDelay={self.cmdDelay})"

  '''
  @param cmdDelay Dauer Wert * Zeitbasis [ms].
  '''
  def getCmdDelay(self):
    return self.cmdDelay






import pyhausbus.HausBusUtils as HausBusUtils

class On:
  CLASS_ID = 20
  FUNCTION_ID = 3

  def __init__(self,brightness:int, duration:int, onDelay:int):
    self.brightness=brightness
    self.duration=duration
    self.onDelay=onDelay


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return On(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"On(brightness={self.brightness}, duration={self.duration}, onDelay={self.onDelay})"

  '''
  @param brightness 0-100% Helligkeit.
  '''
  def getBrightness(self):
    return self.brightness

  '''
  @param duration Einschaltdauer in Sekunden.
  '''
  def getDuration(self):
    return self.duration

  '''
  @param onDelay Einschaltverzoegerung: Wert * Zeitbasis [ms]\r\n0=Keine.
  '''
  def getOnDelay(self):
    return self.onDelay






from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.params.EBaudrate import EBaudrate
from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.params.EDataSetting import EDataSetting
import pyhausbus.HausBusUtils as HausBusUtils

class Configuration:
  CLASS_ID = 45
  FUNCTION_ID = 128

  def __init__(self,baudrate:EBaudrate, dataSetting:EDataSetting, responseTimeout:int):
    self.baudrate=baudrate
    self.dataSetting=dataSetting
    self.responseTimeout=responseTimeout


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Configuration(EBaudrate._fromBytes(dataIn, offset), EDataSetting._fromBytes(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"Configuration(baudrate={self.baudrate}, dataSetting={self.dataSetting}, responseTimeout={self.responseTimeout})"

  '''
  @param baudrate Verbindungsgeschwindigkeit.
  '''
  def getBaudrate(self):
    return self.baudrate

  '''
  @param dataSetting Anzahl Daten-Bits.
  '''
  def getDataSetting(self):
    return self.dataSetting

  '''
  @param responseTimeout Zeit in [ms] um auf eine Antwort zu warten.
  '''
  def getResponseTimeout(self):
    return self.responseTimeout






from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils

class ModbusSlave(ABusFeature):
  CLASS_ID:int = 14

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return ModbusSlave(HausBusUtils.getObjectId(deviceId, 14, instanceId))





import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EDataSetting(Enum):
  8N1=128
  8E1=132
  8O1=136
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EDataSetting.__members__.values():
      if (act.value == checkValue):
        return act

    return EDataSetting.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EDataSetting':
    try:
      return EDataSetting[name]
    except KeyError:
      return EDataSetting.SER_UNKNOWN 







import pyhausbus.HausBusUtils as HausBusUtils

class EvInterrupt:
  CLASS_ID = 90
  FUNCTION_ID = 211

  def __init__(self,value:int, stamp:int):
    self.value=value
    self.stamp=stamp


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvInterrupt(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToDWord(dataIn, offset))

  def __str__(self):
    return f"EvInterrupt(value={self.value}, stamp={self.stamp})"

  '''
  @param value .
  '''
  def getValue(self):
    return self.value

  '''
  @param stamp .
  '''
  def getStamp(self):
    return self.stamp






from pyhausbus.de.hausbus.homeassistant.proxy.pIDController.params.EEnable import EEnable
import pyhausbus.HausBusUtils as HausBusUtils

class Enable:
  CLASS_ID = 44
  FUNCTION_ID = 3

  def __init__(self,enable:EEnable):
    self.enable=enable


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Enable(EEnable._fromBytes(dataIn, offset))

  def __str__(self):
    return f"Enable(enable={self.enable})"

  '''
  @param enable Reglerverhalten ein/ausschalten.
  '''
  def getEnable(self):
    return self.enable






import pyhausbus.HausBusUtils as HausBusUtils

class ReadMemory:
  CLASS_ID = 0
  FUNCTION_ID = 7

  def __init__(self,address:int, length:int):
    self.address=address
    self.length=length


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return ReadMemory(HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"ReadMemory(address={self.address}, length={self.length})"

  '''
  @param address .
  '''
  def getAddress(self):
    return self.address

  '''
  @param length .
  '''
  def getLength(self):
    return self.length






import pyhausbus.HausBusUtils as HausBusUtils

class Current:
  CLASS_ID = 90
  FUNCTION_ID = 128

  def __init__(self,current:int):
    self.current=current


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Current(HausBusUtils.bytesToDWord(dataIn, offset))

  def __str__(self):
    return f"Current(current={self.current})"

  '''
  @param current verbrauchter Strom in Wattstunden.
  '''
  def getCurrent(self):
    return self.current






import pyhausbus.HausBusUtils as HausBusUtils

class ObjectId:
  objectId:int=0

  def __init__ (self, objectId:int):
    self.objectId = objectId

  def getValue(self) -> int:
    return self.objectId

  def getDeviceId(self) -> int:
    return HausBusUtils.getDeviceId(self.objectId)

  def getClassId(self) -> int:
    return HausBusUtils.getClassId(self.objectId)

  def getInstanceId(self) -> int:
    return HausBusUtils.getInstanceId(self.objectId)

  def __str__(self):
    return f"Object(DeviceId={self.getDeviceId()}, ClassId={self.getClassId()}, InstanceId={self.getInstanceId()} - {hex(self.objectId)} ({self.objectId}))"



from abc import ABC, abstractmethod
from pyhausbus.ABusFeature import ABusFeature
from pyhausbus.de.hausbus.homeassistant.proxy.controller.data.ModuleId import ModuleId
from pyhausbus.de.hausbus.homeassistant.proxy.controller.data.Configuration import Configuration

class IBusDeviceListener(ABC):

    @abstractmethod
    def newDeviceDetected(self,device_id:int, model_type: str, module_id: ModuleId, configuration: Configuration, channels: list[ABusFeature]):
        pass


import pyhausbus.HausBusUtils as HausBusUtils

class WriteMemory:
  CLASS_ID = 0
  FUNCTION_ID = 8

  def __init__(self,address:int, data:bytearray):
    self.address=address
    self.data=data


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return WriteMemory(HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToBlob(dataIn, offset))

  def __str__(self):
    return f"WriteMemory(address={self.address}, data={self.data})"

  '''
  @param address .
  '''
  def getAddress(self):
    return self.address

  '''
  @param data .
  '''
  def getData(self):
    return self.data






import pyhausbus.HausBusUtils as HausBusUtils

class EvAbove:
  CLASS_ID = 42
  FUNCTION_ID = 202

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvAbove()

  def __str__(self):
    return f"EvAbove()"






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EFunction(Enum):
  READ_COILS=1
  READ_DISCRETE_INPUTS=2
  READ_HOLDING_REGISTERS=3
  READ_INPUT_REGISTERS=4
  WRITE_SINGLE_COIL=5
  WRITE_SINGLE_REGISTER=6
  WRITE_MULTIPLE_COILS=15
  WRITE_MULTIPLE_REGISTERS=16
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EFunction.__members__.values():
      if (act.value == checkValue):
        return act

    return EFunction.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EFunction':
    try:
      return EFunction[name]
    except KeyError:
      return EFunction.SER_UNKNOWN 







import pyhausbus.HausBusUtils as HausBusUtils

class EvClosed:
  CLASS_ID = 18
  FUNCTION_ID = 200

  def __init__(self,position:int):
    self.position=position


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvClosed(HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"EvClosed(position={self.position})"

  '''
  @param position in Prozent.
  '''
  def getPosition(self):
    return self.position






from pyhausbus.de.hausbus.homeassistant.proxy.led.params.MOptions import MOptions
import pyhausbus.HausBusUtils as HausBusUtils

class Configuration:
  CLASS_ID = 21
  FUNCTION_ID = 128

  def __init__(self,dimmOffset:int, minBrightness:int, timeBase:int, options:MOptions):
    self.dimmOffset=dimmOffset
    self.minBrightness=minBrightness
    self.timeBase=timeBase
    self.options=options


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Configuration(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset), MOptions._fromBytes(dataIn, offset))

  def __str__(self):
    return f"Configuration(dimmOffset={self.dimmOffset}, minBrightness={self.minBrightness}, timeBase={self.timeBase}, options={self.options})"

  '''
  @param dimmOffset 0-100% offset auf den im Kommando angegebenen Helligkeitswert.
  '''
  def getDimmOffset(self):
    return self.dimmOffset

  '''
  @param minBrightness Eine ausgeschaltete LED leuchtet immer noch mit dieser Helligkeit 0-100%.
  '''
  def getMinBrightness(self):
    return self.minBrightness

  '''
  @param timeBase Zeitbasis [ms] f?  ? ? ?r Zeitabh?  ? ? ??ngige Befehle..
  '''
  def getTimeBase(self):
    return self.timeBase

  '''
  @param options Reservierte Bits muessen immer deaktiviert sein. Das Aktivieren eines reservierten Bits fuehrt nach dem Neustart des Controllers zu den Standart-Einstellungen..
  '''
  def getOptions(self) -> MOptions:
    return self.options






from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.params.EErrorCode import EErrorCode
from pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.params.EState import EState
from pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.data.LastData import LastData
from pyhausbus.de.hausbus.homeassistant.proxy.rFIDReader.data.State import State

class RFIDReader(ABusFeature):
  CLASS_ID:int = 43

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return RFIDReader(HausBusUtils.getObjectId(deviceId, 43, instanceId))

  """
  """
  def evConnected(self):
    LOGGER.debug("evConnected")
    hbCommand = HausBusCommand(self.objectId, 200, "evConnected")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param errorCode .
  """
  def evError(self, errorCode:EErrorCode):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def setConfiguration(self):
    LOGGER.debug("setConfiguration")
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param state State of the RFID-Reader hardware.
  """
  def State(self, state:EState):
    LOGGER.debug("State"+" state = "+str(state))
    hbCommand = HausBusCommand(self.objectId, 129, "State")
    hbCommand.addByte(state.value)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param tagID ID of the detected RFID tag.
  """
  def evData(self, tagID:int):
    LOGGER.debug("evData"+" tagID = "+str(tagID))
    hbCommand = HausBusCommand(self.objectId, 201, "evData")
    hbCommand.addDWord(tagID)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getLastData(self):
    LOGGER.debug("getLastData")
    hbCommand = HausBusCommand(self.objectId, 3, "getLastData")
    ResultWorker()._setResultInfo(LastData,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param tagID last tagID read successfully.
  """
  def LastData(self, tagID:int):
    LOGGER.debug("LastData"+" tagID = "+str(tagID))
    hbCommand = HausBusCommand(self.objectId, 130, "LastData")
    hbCommand.addDWord(tagID)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def Configuration(self):
    LOGGER.debug("Configuration")
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getState(self):
    LOGGER.debug("getState")
    hbCommand = HausBusCommand(self.objectId, 2, "getState")
    ResultWorker()._setResultInfo(State,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





import pyhausbus.HausBusUtils as HausBusUtils

class MemoryData:
  CLASS_ID = 0
  FUNCTION_ID = 132

  def __init__(self,address:int, data:bytearray):
    self.address=address
    self.data=data


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return MemoryData(HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToBlob(dataIn, offset))

  def __str__(self):
    return f"MemoryData(address={self.address}, data={self.data})"

  '''
  @param address Adresse des gemeldeten Speicherinhaltes.
  '''
  def getAddress(self):
    return self.address

  '''
  @param data Daten....
  '''
  def getData(self):
    return self.data






from pyhausbus.BusHandler import BusHandler
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.HausBusUtils import LOGGER
from pyhausbus.ObjectId import ObjectId

class HausBusCommand:

  def __init__ (self, receiverObjectId:int, functionId:int, debug:str):
    self.commandCounter:int = 0
    self.receiverObjectId = receiverObjectId
    self.debug = debug
    self.busHandler = BusHandler.getInstance()
    self.data:bytearray=bytearray()
    self.params:bytearray=bytearray()
    self.bytes:bytearray=None

    # Kontroll-Byte
    self.data.append(0)

    # Nachrichtenzaehler
    self.data.append(0)

    # Sender-ID
    for act in HausBusUtils.dWordToBytes(HausBusUtils.HOMESERVER_OBJECT_ID):
      self.data.append(act)

    # Empfaenger-ID
    for act in HausBusUtils.dWordToBytes(receiverObjectId):
      self.data.append(act)

    self.params.append(functionId)

  def addByte(self, byteParam:int):
    self.params.append(byteParam)

  def addSByte(self, sByteParam:int):
    if (sByteParam < 0):
      self.params.append((256 + sByteParam))
    else:
      self.params.append(sByteParam)

  def addBlob(self, blob:bytearray):
    for act in blob:
      self.params.append(act)

  def addString(self, value:str):
    for i in range(len(value)):
      self.params.append(value[i])
    self.params.append(0)

  def addMap(self, inMap):
    for key, value in inMap.items():
      self.addByte(key)
      self.addByte(value)

  def addWord(self, wordParam:int):
    HausBusUtils.addWord(wordParam, self.params)

  def addDWord(self, dWordParam:int):
    HausBusUtils.addDword(dWordParam, self.params)

  def _createBytes(self):

    # Datenlaenge
    for act in HausBusUtils.wordToBytes(len(self.params)):
      self.data.append(act)

    # Daten
    for act in self.params:
      self.data.append(act)

    self.bytes = bytearray(len(self.data))
    for i in range (len(self.data)):
      self.bytes[i] = self.data[i]

  def send(self):
    LOGGER.debug("Sende " + self.debug + " an " + str(ObjectId(self.receiverObjectId)))
    if (self.bytes == None):
      self._createBytes()

    return self.busHandler.sendData(self.bytes, self.debug + " to " + str(HausBusUtils.getDeviceId(self.receiverObjectId)))

  def __str__(self):
    return f"HausBusCommand(debug={self.debug} to {HausBusUtils.formatObjectId(self.receiverObjectId)}, params={self.params}, data={self.data}"


import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EBaudrate(Enum):
  _1200=0
  _2400=1
  _4800=2
  _9600=3
  _19200=4
  _38400=5
  _57600=6
  _115200=7
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EBaudrate.__members__.values():
      if (act.value == checkValue):
        return act

    return EBaudrate.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EBaudrate':
    try:
      return EBaudrate[name]
    except KeyError:
      return EBaudrate.SER_UNKNOWN 







import logging
import time
import traceback

LOGGER = logging.getLogger("pyhausbus")

HOMESERVER_DEVICE_ID: int = 9998
HOMESERVER_OBJECT_ID: int = (HOMESERVER_DEVICE_ID << 16) + (0 << 8) + 1

UDP_PORT = 9 #5855


def getObjectId(deviceId: int, classId: int, instanceId: int) -> int:
    return (deviceId << 16) + (classId << 8) + instanceId


def bytesToDWord(data: bytearray, offset) -> int:
    try:
        result = 0
        result += data[offset[0]] & 0xFF
        offset[0] += 1

        result += (data[offset[0]] & 0xFF) * 256
        offset[0] += 1

        result += (data[offset[0]] & 0xFF) * 65536
        offset[0] += 1

        result += (data[offset[0]] & 0xFF) * 16777216
        offset[0] += 1

        return result
    except Exception as err:
        LOGGER.debug(f"error: {err}", exc_info=True, stack_info=True)
        return 0


def bytesToWord(data: bytearray, offset) -> int:
    try:
        result = 0
        result += data[offset[0]] & 0xFF
        offset[0] += 1

        result += (data[offset[0]] & 0xFF) * 256
        offset[0] += 1

        return result
    except Exception as err:
        LOGGER.debug(f"error: {err}", exc_info=True, stack_info=True)
        return 0


def bytesToInt(data: bytearray, offset) -> int:
    try:
        if len(data) <= offset[0]:
            return 0
        result = data[offset[0]] & 0xFF
        offset[0] += 1
        return result
    except Exception as err:
        LOGGER.debug(f"error: {err}", exc_info=True, stack_info=True)
        return 0


def bytesToString(data: bytearray, offset) -> str:
    try:
        result = ""
        for i in range(offset[0], len(data)):
            offset[0] += 1
            if data[i] == 0:
                break

            result += chr(data[i])
        return result
    except Exception as err:
        LOGGER.debug(f"error: {err}", exc_info=True, stack_info=True)
        return ""


def bytesToDebugString(message: bytearray) -> str:
    result = ""
    for byte in message:
        if result != "":
            result = result + ", "
        result = result + hex(byte)
    return result


def getInstanceId(objectId) -> int:
    return (objectId) & 0xFF


def getDeviceId(objectId) -> int:
    return ((objectId >> 24) & 0xFF) * 256 + ((objectId >> 16) & 0xFF)


def getClassId(objectId) -> int:
    return (objectId >> 8) & 0xFF


def formatObjectId(objectId) -> str:
    result = "[DeviceId: " + str(getDeviceId(objectId))
    result += ", ClassId: " + str(getClassId(objectId))
    result += ", Instance: " + str(getInstanceId(objectId)) + "]"
    return result


def getClockIndependMillis():
    return time.perf_counter() * 1000


def setBit(is_set: bool, bit: int, value: int) -> int:
    if is_set:
        value |= 1 << bit
    else:
        value &= ~(1 << bit)
    return value


def isBitSet(bit: int, value: int) -> bool:
    return ((value >> bit) & 1) > 0


def bytesToBlob(data: bytearray, offset) -> bytearray:
    try:
        result = bytearray(len(data) - offset[0])
        result[:] = data[offset[0] :]
        offset[0] = offset[0] + len(data)
        return result
    except Exception as err:
        LOGGER.debug(f"error: {err}", exc_info=True, stack_info=True)
        return 0


def dWordToBytes(value: int) -> bytearray:
    result = bytearray(4)
    result[0] = value & 0xFF
    result[1] = (value >> 8) & 0xFF
    result[2] = (value >> 16) & 0xFF
    result[3] = (value >> 24) & 0xFF
    return result


def addWord(value: int, inOutList):
    inOutList.append(value & 0xFF)
    inOutList.append((value >> 8) & 0xFF)


def addDword(value: int, inOutList):
    inOutList.append(value & 0xFF)
    inOutList.append((value >> 8) & 0xFF)
    inOutList.append((value >> 16) & 0xFF)
    inOutList.append((value >> 24) & 0xFF)


def wordToBytes(value: int) -> bytearray:
    result = bytearray(2)
    result[0] = value & 0xFF
    result[1] = (value >> 8) & 0xFF
    return result


def bytesToSInt(data: bytearray, offset) -> int:
    result = data[offset[0]] & 0xFF
    offset[0] += 1
    if result > 127:
        result -= 256
    return result


def formatBytes(
    data: bytearray, offset: int = 0, length: int = -1, asHex: bool = True
) -> str:
    if length == -1:
        length = len(data)

        if data == None:
            return "NULL"

        result = ""

        for i in range(offset, length):
            if asHex:
                result += hex(data[i])
            else:
                result += data[i] & 0xFF
        return result


def bytesToList(data: bytearray, offset):
    try:
        result = bytearray()
        offsetStart = offset[0]
        for i in range(offsetStart, len(data), 2):
            key = data[i] & 0xFF
            value = data[i + 1] & 0xFF
            result.append(key)
            result.append(value)
            offset[0] += 2
        return result
    except Exception as err:
        LOGGER.debug(f"error: {err}", exc_info=True, stack_info=True)
        return bytearray()



import pyhausbus.HausBusUtils as HausBusUtils

class Exec:
  CLASS_ID = 1
  FUNCTION_ID = 0

  def __init__(self,command:str):
    self.command=command


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Exec(HausBusUtils.bytesToString(dataIn, offset))

  def __str__(self):
    return f"Exec(command={self.command})"

  '''
  @param command .
  '''
  def getCommand(self):
    return self.command






import pyhausbus.HausBusUtils as HausBusUtils

class MinIdleTime:
  CLASS_ID = 176
  FUNCTION_ID = 129

  def __init__(self,time_ms:int):
    self.time_ms=time_ms


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return MinIdleTime(HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"MinIdleTime(time_ms={self.time_ms})"

  '''
  @param time_ms Mindestwartezeit [ms].
  '''
  def getTime_ms(self):
    return self.time_ms






import pyhausbus.HausBusUtils as HausBusUtils

class Status:
  CLASS_ID = 21
  FUNCTION_ID = 129

  def __init__(self,brightness:int, duration:int):
    self.brightness=brightness
    self.duration=duration


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Status(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"Status(brightness={self.brightness}, duration={self.duration})"

  '''
  @param brightness Helligkeit der LED.
  '''
  def getBrightness(self):
    return self.brightness

  '''
  @param duration Einschaltdauer: Wert * Zeitbasis [ms]\r\n0=Endlos.
  '''
  def getDuration(self):
    return self.duration






import pyhausbus.HausBusUtils as HausBusUtils

class GetRegisterConfiguration:
  CLASS_ID = 45
  FUNCTION_ID = 2

  def __init__(self,idx:int):
    self.idx=idx


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return GetRegisterConfiguration(HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"GetRegisterConfiguration(idx={self.idx})"

  '''
  @param idx index of the configuration slot.
  '''
  def getIdx(self):
    return self.idx






import pyhausbus.HausBusUtils as HausBusUtils

class ReadRules:
  CLASS_ID = 0
  FUNCTION_ID = 10

  def __init__(self,offset:int, length:int):
    self.offset=offset
    self.length=length


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return ReadRules(HausBusUtils.bytesToWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"ReadRules(offset={self.offset}, length={self.length})"

  '''
  @param offset Offset im Gesamtregelblock.
  '''
  def getOffset(self):
    return self.offset

  '''
  @param length Datenlaenge.
  '''
  def getLength(self):
    return self.length






from pyhausbus.HausBusCommand import HausBusCommand
from pyhausbus.ABusFeature import *
from pyhausbus.ResultWorker import ResultWorker
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.data.RegisterConfiguration import RegisterConfiguration
from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.params.EType import EType
from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.params.EFunction import EFunction
from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.params.EBaudrate import EBaudrate
from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.params.EDataSetting import EDataSetting
from pyhausbus.de.hausbus.homeassistant.proxy.modBusMaster.params.EErrorCode import EErrorCode

class ModBusMaster(ABusFeature):
  CLASS_ID:int = 45

  def __init__ (self,objectId:int):
    super().__init__(objectId)

  @staticmethod
  def create(deviceId:int, instanceId:int):
    return ModBusMaster(HausBusUtils.getObjectId(deviceId, 45, instanceId))

  """
  @param idx index of the configuration slot.
  """
  def getRegisterConfiguration(self, idx:int):
    LOGGER.debug("getRegisterConfiguration"+" idx = "+str(idx))
    hbCommand = HausBusCommand(self.objectId, 2, "getRegisterConfiguration")
    hbCommand.addByte(idx)
    ResultWorker()._setResultInfo(RegisterConfiguration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param idx index of the configuration slot.
  @param node Geraeteadresse im ModBus.
  @param type Unterstuetzte Register Typen.
  @param address Register Adresse.
  """
  def setRegisterConfiguration(self, idx:int, node:int, type:EType, address:int):
    LOGGER.debug("setRegisterConfiguration"+" idx = "+str(idx)+" node = "+str(node)+" type = "+str(type)+" address = "+str(address))
    hbCommand = HausBusCommand(self.objectId, 3, "setRegisterConfiguration")
    hbCommand.addByte(idx)
    hbCommand.addByte(node)
    hbCommand.addByte(type.value)
    hbCommand.addWord(address)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param idx index of the configuration slot.
  @param node device node on ModBus.
  @param type Unterstuetzte Register Typen.
  @param address Register Adresse.
  """
  def RegisterConfiguration(self, idx:int, node:int, type:EType, address:int):
    LOGGER.debug("RegisterConfiguration"+" idx = "+str(idx)+" node = "+str(node)+" type = "+str(type)+" address = "+str(address))
    hbCommand = HausBusCommand(self.objectId, 129, "RegisterConfiguration")
    hbCommand.addByte(idx)
    hbCommand.addByte(node)
    hbCommand.addByte(type.value)
    hbCommand.addWord(address)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param node Bus-Knoten Geraete-Adresse.
  @param function Mod-Bus Funktion.
  @param address Adresse in Geraet.
  @param data Daten.
  """
  def genericResponse(self, node:int, function:EFunction, address:int, data:bytearray):
    LOGGER.debug("genericResponse"+" node = "+str(node)+" function = "+str(function)+" address = "+str(address)+" data = "+str(data))
    hbCommand = HausBusCommand(self.objectId, 130, "genericResponse")
    hbCommand.addByte(node)
    hbCommand.addByte(function.value)
    hbCommand.addWord(address)
    hbCommand.addBlob(data)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param node Bus-Knoten Geraete-Adresse.
  @param function Mod-Bus Funktion.
  @param address Adresse in Geraet.
  @param data Daten.
  """
  def genericCommand(self, node:int, function:EFunction, address:int, data:bytearray):
    LOGGER.debug("genericCommand"+" node = "+str(node)+" function = "+str(function)+" address = "+str(address)+" data = "+str(data))
    hbCommand = HausBusCommand(self.objectId, 4, "genericCommand")
    hbCommand.addByte(node)
    hbCommand.addByte(function.value)
    hbCommand.addWord(address)
    hbCommand.addBlob(data)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  """
  def getConfiguration(self):
    LOGGER.debug("getConfiguration")
    hbCommand = HausBusCommand(self.objectId, 0, "getConfiguration")
    ResultWorker()._setResultInfo(Configuration,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param baudrate Verbindungsgeschwindigkeit.
  @param dataSetting Anzahl Daten-Bits.
  @param responseTimeout Zeit in [ms] um auf eine Antwort zu warten.
  """
  def setConfiguration(self, baudrate:EBaudrate, dataSetting:EDataSetting, responseTimeout:int):
    LOGGER.debug("setConfiguration"+" baudrate = "+str(baudrate)+" dataSetting = "+str(dataSetting)+" responseTimeout = "+str(responseTimeout))
    hbCommand = HausBusCommand(self.objectId, 1, "setConfiguration")
    hbCommand.addByte(baudrate.value)
    hbCommand.addByte(dataSetting.value)
    hbCommand.addWord(responseTimeout)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param baudrate Verbindungsgeschwindigkeit.
  @param dataSetting Anzahl Daten-Bits.
  @param responseTimeout Zeit in [ms] um auf eine Antwort zu warten.
  """
  def Configuration(self, baudrate:EBaudrate, dataSetting:EDataSetting, responseTimeout:int):
    LOGGER.debug("Configuration"+" baudrate = "+str(baudrate)+" dataSetting = "+str(dataSetting)+" responseTimeout = "+str(responseTimeout))
    hbCommand = HausBusCommand(self.objectId, 128, "Configuration")
    hbCommand.addByte(baudrate.value)
    hbCommand.addByte(dataSetting.value)
    hbCommand.addWord(responseTimeout)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")

  """
  @param errorCode .
  @param data .
  """
  def evError(self, errorCode:EErrorCode, data:int):
    LOGGER.debug("evError"+" errorCode = "+str(errorCode)+" data = "+str(data))
    hbCommand = HausBusCommand(self.objectId, 255, "evError")
    hbCommand.addByte(errorCode.value)
    hbCommand.addByte(data)
    ResultWorker()._setResultInfo(None,self.getObjectId())
    hbCommand.send()
    LOGGER.debug("returns")





from pyhausbus.de.hausbus.homeassistant.proxy.taster.params.EState import EState
import pyhausbus.HausBusUtils as HausBusUtils

class EvHoldStart:
  CLASS_ID = 16
  FUNCTION_ID = 203

  def __init__(self,state:EState):
    self.state=state


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvHoldStart(EState._fromBytes(dataIn, offset))

  def __str__(self):
    return f"EvHoldStart(state={self.state})"

  '''
  @param state .
  '''
  def getState(self):
    return self.state






import pyhausbus.HausBusUtils as HausBusUtils

class EvGroupOn:
  CLASS_ID = 0
  FUNCTION_ID = 203

  def __init__(self,index:int, status:int):
    self.index=index
    self.status=status


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvGroupOn(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"EvGroupOn(index={self.index}, status={self.status})"

  '''
  @param index Gruppenindex.
  '''
  def getIndex(self):
    return self.index

  '''
  @param status Status der Bits in der logischen Gruppe..
  '''
  def getStatus(self):
    return self.status






import pyhausbus.HausBusUtils as HausBusUtils

class AnnounceServer:
  CLASS_ID = 91
  FUNCTION_ID = 1

  def __init__(self,IP0:int, IP1:int, IP2:int, IP3:int, port:int):
    self.IP0=IP0
    self.IP1=IP1
    self.IP2=IP2
    self.IP3=IP3
    self.port=port


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return AnnounceServer(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"AnnounceServer(IP0={self.IP0}, IP1={self.IP1}, IP2={self.IP2}, IP3={self.IP3}, port={self.port})"

  '''
  @param IP0 .
  '''
  def getIP0(self):
    return self.IP0

  '''
  @param IP1 .
  '''
  def getIP1(self):
    return self.IP1

  '''
  @param IP2 .
  '''
  def getIP2(self):
    return self.IP2

  '''
  @param IP3 .
  '''
  def getIP3(self):
    return self.IP3

  '''
  @param port .
  '''
  def getPort(self):
    return self.port






import pyhausbus.HausBusUtils as HausBusUtils

class SetVariable:
  CLASS_ID = 1
  FUNCTION_ID = 126

  def __init__(self,varId:int, varValue:int):
    self.varId=varId
    self.varValue=varValue


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return SetVariable(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToInt(dataIn, offset))

  def __str__(self):
    return f"SetVariable(varId={self.varId}, varValue={self.varValue})"

  '''
  @param varId .
  '''
  def getVarId(self):
    return self.varId

  '''
  @param varValue .
  '''
  def getVarValue(self):
    return self.varValue






import pyhausbus.HausBusUtils as HausBusUtils

class EvOn:
  CLASS_ID = 19
  FUNCTION_ID = 201

  def __init__(self,duration:int):
    self.duration=duration


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvOn(HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"EvOn(duration={self.duration})"

  '''
  @param duration Dauer.
  '''
  def getDuration(self):
    return self.duration






import importlib

from pyhausbus.ABusFeature import ABusFeature
from pyhausbus.BusHandler import BusHandler
from pyhausbus.HausBusUtils import HOMESERVER_DEVICE_ID
from pyhausbus.HausBusUtils import LOGGER
import pyhausbus.HausBusUtils as HausBusUtils
from pyhausbus.IBusDataListener import IBusDataListener
from pyhausbus.IBusDeviceListener import IBusDeviceListener
from pyhausbus.ObjectId import ObjectId
from pyhausbus.ResultWorker import ResultWorker
from pyhausbus.Templates import Templates
from pyhausbus.de.hausbus.homeassistant.proxy.Controller import Controller
import pyhausbus.de.hausbus.homeassistant.proxy.ProxyFactory as ProxyFactory
from pyhausbus.de.hausbus.homeassistant.proxy.controller.data.Configuration import Configuration
from pyhausbus.de.hausbus.homeassistant.proxy.controller.data.EvStarted import EvStarted
from pyhausbus.de.hausbus.homeassistant.proxy.controller.data.ModuleId import ModuleId
from pyhausbus.de.hausbus.homeassistant.proxy.controller.data.RemoteObjects import  RemoteObjects
from pyhausbus.de.hausbus.homeassistant.proxy.controller.params.EIndex import EIndex
import threading
import queue
import time
import logging
import traceback

_module_cache = {}
_class_cache = {}


class HomeServer(IBusDataListener):
    _instance = None
    bushandler = None
    device_listeners = []

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self):
        LOGGER.debug("init homeserver")
        self.bushandler = BusHandler.getInstance()
        self.bushandler.addBusEventListener(ResultWorker())
        self.bushandler.addBusEventListener(self)
        self._receivedSomething = False
        self.module_ids: dict[int, ModuleId] = {}
        self.configurations: dict[int, Configuration] = {}
        self.remote_objects: dict[int, RemoteObjects] = {}
        self.worker = DeviceWorker(self)
        self.worker.start()
        self.collector = DeviceCollector(self.worker, timeout=1.0)
        self.collector.start()
        self.known_devices = set()

    def searchDevices(self):
        controller = Controller(0)
        controller.getModuleId(EIndex.RUNNING)

    def addBusEventListener(self, listener: IBusDataListener):
        self.bushandler.addBusEventListener(listener)

    def removeBusEventListener(self, listener: IBusDataListener):
        self.bushandler.removeBusEventListener(listener)

    def addBusDeviceListener(self, listener: IBusDeviceListener):
        if not listener in self.device_listeners:
          self.device_listeners.append(listener)

    def removeBusDeviceListener(self, listener: IBusDeviceListener):
        self.device_listeners.remove(listener)

    def is_any_device_found(self) -> bool:
        return self._receivedSomething

    def is_internal_device(self, deviceId:int) -> bool:
        # if deviceId in [110, 503, 1000, 1541, 3422, 4000, 4001, 4002, 4003, 4004, 4005, 4009, 4096, 5068, 8192, 8270, 11581, 12223, 12622, 13976, 14896, 18343, 19075, 20043, 21336, 22909, 24261, 25661, 25874, 28900, 3423, 4006, 4008]:
         #   return True
        return deviceId in {HOMESERVER_DEVICE_ID, 9999, 12222}

    def get_configuration_from_cache(self, device_id: int) -> Configuration:

        configuration = self.configurations.get(device_id)

        if configuration is None:
          LOGGER.debug("reading configuration for cache for {device_id}")
          configuration = Controller.create(device_id, 1).getConfiguration()

        return configuration

    def get_module_id_from_cache(self, device_id: int) -> Configuration:

        module_id = self.module_ids.get(device_id)

        if module_id is None:
          LOGGER.debug("reading module_id for cache for {device_id}")
          module_id = Controller.create(device_id, 1).getModuleId()

        return module_id

    def get_model(self, device_id: int) -> str:

        configuration = self.get_configuration_from_cache(device_id)
        fcke = configuration.getFCKE()
        # special_type = configuration.getStartupDelay()

        firmware_id = self.get_module_id_from_cache(device_id).getFirmwareId()

        model = Templates.get_instance().getModuleName(firmware_id, fcke)

        LOGGER.debug(f"device_id {device_id} fcke {fcke} firmwareId {firmware_id} is model {model}")
        return model

    def busDataReceived(self, busDataMessage):
        """if a device restarts during runtime, we automatically read moduleId"""

        device_id = ObjectId(busDataMessage.getSenderObjectId()).getDeviceId()

        if self.is_internal_device(device_id):
          return

        # ignore own messages
        if not device_id == HOMESERVER_DEVICE_ID:
          self._receivedSomething = True

        if isinstance(busDataMessage.getData(), ModuleId):
            self.module_ids[device_id] = busDataMessage.getData()
            self.known_devices.add(device_id)
            self.collector.add_response(device_id)
            # LOGGER.debug("auto calling getConfiguration")
            # Controller(busDataMessage.getSenderObjectId()).getConfiguration()

        if not device_id in self.known_devices:
          LOGGER.debug(f"got message from unknown device {device_id}. reading module_id");
          Controller.create(device_id, 1).getModuleId(EIndex.RUNNING)

        if isinstance(busDataMessage.getData(), Configuration):
            self.configurations[device_id] = busDataMessage.getData()
            # LOGGER.debug("auto calling getRemoteObjects")
            # Controller(busDataMessage.getSenderObjectId()).getRemoteObjects()

        if isinstance(busDataMessage.getData(), RemoteObjects):
            self.remote_objects[device_id] = busDataMessage.getData()

        """ if a device restarts during runtime, we automatically read moduleId"""
        if isinstance(busDataMessage.getData(), EvStarted):
            LOGGER.debug("auto calling getModuleId")
            Controller(busDataMessage.getSenderObjectId()).getModuleId(EIndex.RUNNING)


# --- Worker-Thread-Klasse ---
class DeviceWorker(threading.Thread):

    def __init__(self, homeserver):
        super().__init__(daemon=True)
        self.queue = queue.Queue()
        self.running = True
        self.homeserver = homeserver

    def run(self):
        timeout = 3  # Sekunden
        LOGGER.debug("Worker gestartet.")
        while self.running:
            try:
                # Warte auf neues Device
                device_id = self.queue.get(timeout=1)
            except queue.Empty:
                continue

            try:
                module_Id = self.homeserver.module_ids[device_id]
                if module_Id is not None:
                  LOGGER.debug(f"[DeviceWorker {device_id}] Working with module_id {module_Id}")
                  self.homeserver.configurations[device_id] = None
                  Controller.create(device_id, 1).getConfiguration()

                  for _ in range(2):
                      start_time = time.time()
                      while self.homeserver.configurations.get(device_id) is None:
                          if time.time() - start_time > timeout:
                              LOGGER.debug(f"[DeviceWorker {device_id}] Timeout for configuration")
                              break

                          time.sleep(0.1)  # kurze Pause, um CPU nicht zu blockieren

                  configuration = self.homeserver.configurations.get(device_id)
                  if configuration is not None:
                    LOGGER.debug(f"[DeviceWorker {device_id}] got configuration. reading remoteobjects")
                    self.homeserver.remote_objects[device_id] = None
                    Controller.create(device_id, 1).getRemoteObjects()

                    for _ in range(2):
                        start_time = time.time()
                        while self.homeserver.remote_objects.get(device_id) is None:
                          if time.time() - start_time > timeout:
                            LOGGER.debug(f"[DeviceWorker {device_id}] Timeout for remote objects")
                            break

                          time.sleep(0.1)  # kurze Pause, um CPU nicht zu blockieren

                    remote_objects = self.homeserver.remote_objects.get(device_id)
                    if remote_objects is not None:
                      LOGGER.debug(f"[DeviceWorker {device_id}] discovery ok")
                      instances = self.getHomeassistantChannels(device_id, remote_objects)
                      for actListener in self.homeserver.device_listeners:
                        actListener.newDeviceDetected(device_id, self.homeserver.get_model(device_id), module_Id, configuration, instances)

            except Exception as e:
                logging.error("[DeviceWorker {device_id}] error for %s %s\n%s", e, device_id, traceback.format_exc())
            finally:
                self.queue.task_done()

        LOGGER.debug("[DeviceWorker]  finished.")

    def getHomeassistantChannels(self, device_id: int, remoteObjects: RemoteObjects):

        firmware_id = self.homeserver.get_module_id_from_cache(device_id).getFirmwareId()
        fcke = self.homeserver.get_configuration_from_cache(device_id).getFCKE()
        instances: list[ABusFeature] = self.getDeviceInstances(device_id, remoteObjects)

        templates = Templates.get_instance()

        for instance in instances:
            instanceObjectId = ObjectId(instance.getObjectId())
            name = templates.get_feature_name_from_template(
                firmware_id,
                fcke,
                instanceObjectId.getClassId(),
                instanceObjectId.getInstanceId(),
            )

            LOGGER.debug(
                "name for firmwareId %s, fcke: %s, classId %s, instanceId %s is %s",
                firmware_id,
                fcke,
                instanceObjectId.getClassId(),
                instanceObjectId.getInstanceId(),
                name,
            )

            if name is None:
                className = ProxyFactory.getBusClassNameForClass(
                    instanceObjectId.getClassId()
                ).rsplit(".", 1)[-1]
                name = f"{className} {instanceObjectId.getInstanceId()}"
                LOGGER.debug("generic name %s", name)

            instance.setName(name)

        return instances

    def getDeviceInstances(self, device_id: int, remoteObjects: RemoteObjects):
        objectList = remoteObjects.getObjectList()

        result = []
        for i in range(0, len(objectList), 2):
            instanceId = objectList[i]
            classId = objectList[i + 1]
            className = ProxyFactory.getBusClassNameForClass(classId)
            objectId = HausBusUtils.getObjectId(device_id, classId, instanceId)

            try:
                module_name, class_name = className.rsplit(".", 1)
                
                # Modul aus Cache laden
                if module_name in _module_cache:
                  module = _module_cache[module_name]
                else:
                  module = importlib.import_module(module_name)
                _module_cache[module_name] = module

                # Klasse aus Cache laden
                key = (module_name, class_name)
                if key in _class_cache:
                  cls = _class_cache[key]
                else:
                  cls = getattr(module, class_name)
                _class_cache[key] = cls

                obj = cls(objectId)
                result.append(obj)
                
            except Exception as err:
                LOGGER.error(err, exc_info=True, stack_info=True)
        return result

    def enqueue(self, device_id):
        LOGGER.debug(f"added {device_id} to queue")
        self.queue.put(device_id)

    def stop(self):
        LOGGER.debug("stopping worker")
        self.running = False


# --- Collector, der Geraeteantworten einsammelt ---
class DeviceCollector(threading.Thread):

    def __init__(self, worker, timeout=1.0):
        super().__init__(daemon=True)
        self.worker = worker
        self.timeout = timeout
        self.responses = set()
        self.last_added = 0
        self.lock = threading.Lock()
        self.running = True

    def add_response(self, device_id):
        with self.lock:
            self.responses.add(device_id)
            self.last_added = time.time()
            # Debug
            LOGGER.debug(f"[DeviceCollector] got response from {device_id}")

    def run(self):
        LOGGER.debug("[DeviceCollector] started.")
        while self.running:
            time.sleep(0.1)
            with self.lock:
                # Wenn laenger als timeout keine neue Antwort kam
                if self.responses and (time.time() - self.last_added > self.timeout):
                    devices = list(self.responses)
                    self.responses.clear()
                else:
                    devices = []
            # Ausserhalb des Locks abarbeiten
            for dev in devices:
                LOGGER.debug(f"[DeviceCollector] providing {dev} to worker.")
                self.worker.enqueue(dev)
        LOGGER.debug("[DeviceCollector] finished")

    def stop(self):
        self.running = False



import pyhausbus.HausBusUtils as HausBusUtils

class Status:
  CLASS_ID = 35
  FUNCTION_ID = 129

  def __init__(self,counter:int, fraction:int):
    self.counter=counter
    self.fraction=fraction


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Status(HausBusUtils.bytesToDWord(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"Status(counter={self.counter}, fraction={self.fraction})"

  '''
  @param counter Zaehler ganze Einheiten.
  '''
  def getCounter(self):
    return self.counter

  '''
  @param fraction Bruchteil 0-scaleFactor.
  '''
  def getFraction(self):
    return self.fraction






import pyhausbus.HausBusUtils as HausBusUtils

class EvToggleByDuty:
  CLASS_ID = 19
  FUNCTION_ID = 205

  def __init__(self,duty:int, durationSeconds:int):
    self.duty=duty
    self.durationSeconds=durationSeconds


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return EvToggleByDuty(HausBusUtils.bytesToInt(dataIn, offset), HausBusUtils.bytesToWord(dataIn, offset))

  def __str__(self):
    return f"EvToggleByDuty(duty={self.duty}, durationSeconds={self.durationSeconds})"

  '''
  @param duty 0-100% Pulsverh?ltnis.
  '''
  def getDuty(self):
    return self.duty

  '''
  @param durationSeconds Einschaltdauer in Sekunden.
  '''
  def getDurationSeconds(self):
    return self.durationSeconds






from pyhausbus.de.hausbus.homeassistant.proxy.logicalButton.params.EStatus import EStatus
import pyhausbus.HausBusUtils as HausBusUtils

class Status:
  CLASS_ID = 20
  FUNCTION_ID = 129

  def __init__(self,status:EStatus):
    self.status=status


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return Status(EStatus._fromBytes(dataIn, offset))

  def __str__(self):
    return f"Status(status={self.status})"

  '''
  @param status Zustand der Taster LEDs.
  '''
  def getStatus(self):
    return self.status






from pyhausbus.de.hausbus.homeassistant.proxy.digitalPort.params.EPin import EPin
import pyhausbus.HausBusUtils as HausBusUtils

class SetConfiguration:
  CLASS_ID = 15
  FUNCTION_ID = 1

  def __init__(self,pin0:EPin, pin1:EPin, pin2:EPin, pin3:EPin, pin4:EPin, pin5:EPin, pin6:EPin, pin7:EPin):
    self.pin0=pin0
    self.pin1=pin1
    self.pin2=pin2
    self.pin3=pin3
    self.pin4=pin4
    self.pin5=pin5
    self.pin6=pin6
    self.pin7=pin7


  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return SetConfiguration(EPin._fromBytes(dataIn, offset), EPin._fromBytes(dataIn, offset), EPin._fromBytes(dataIn, offset), EPin._fromBytes(dataIn, offset), EPin._fromBytes(dataIn, offset), EPin._fromBytes(dataIn, offset), EPin._fromBytes(dataIn, offset), EPin._fromBytes(dataIn, offset))

def getPin(self, id:int):
  if id==0:
    return pin0
  if id==1:
    return pin1
  if id==2:
    return pin2
  if id==3:
    return pin3
  if id==4:
    return pin4
  if id==5:
    return pin5
  if id==6:
    return pin6
  if id==7:
    return pin7
  return null

def setPin(self, id:int, pin:EPin):
  if id==0:
    pin0 = pin
  if id==1:
    pin1 = pin
  if id==2:
    pin2 = pin
  if id==3:
    pin3 = pin
  if id==4:
    pin4 = pin
  if id==5:
    pin5 = pin
  if id==6:
    pin6 = pin
  if id==7:
    pin7 = pin


  def __str__(self):
    return f"SetConfiguration(pin0={self.pin0}, pin1={self.pin1}, pin2={self.pin2}, pin3={self.pin3}, pin4={self.pin4}, pin5={self.pin5}, pin6={self.pin6}, pin7={self.pin7})"

  '''
  @param pin0 .
  '''
  def getPin0(self):
    return self.pin0

  '''
  @param pin1 .
  '''
  def getPin1(self):
    return self.pin1

  '''
  @param pin2 .
  '''
  def getPin2(self):
    return self.pin2

  '''
  @param pin3 .
  '''
  def getPin3(self):
    return self.pin3

  '''
  @param pin4 .
  '''
  def getPin4(self):
    return self.pin4

  '''
  @param pin5 .
  '''
  def getPin5(self):
    return self.pin5

  '''
  @param pin6 .
  '''
  def getPin6(self):
    return self.pin6

  '''
  @param pin7 .
  '''
  def getPin7(self):
    return self.pin7






import pyhausbus.HausBusUtils as HausBusUtils

class GetWeather:
  CLASS_ID = 2
  FUNCTION_ID = 5

  @staticmethod
  def _fromBytes(dataIn:bytearray, offset):
    return GetWeather()

  def __str__(self):
    return f"GetWeather()"






import pyhausbus.HausBusUtils as HausBusUtils
from enum import Enum

class EDate(Enum):
  _1=1
  _2=2
  _3=3
  _4=4
  _5=5
  _6=6
  _7=7
  _8=8
  _9=9
  _10=10
  _11=11
  _12=12
  _13=13
  _14=14
  _15=15
  _16=16
  _17=17
  _18=18
  _19=19
  _20=20
  _21=21
  _22=22
  _23=23
  _24=24
  _25=25
  _26=26
  _27=27
  _28=28
  _29=29
  _30=30
  _31=31
  SER_UNKNOWN=-1

  @staticmethod
  def _fromBytes(data:bytearray, offset):
    checkValue = HausBusUtils.bytesToInt(data, offset)
    for act in EDate.__members__.values():
      if (act.value == checkValue):
        return act

    return EDate.SER_UNKNOWN

  @staticmethod
  def value_of(name: str) -> 'EDate':
    try:
      return EDate[name]
    except KeyError:
      return EDate.SER_UNKNOWN 




